@@ -13,6 +13,7 @@ const Tweener = imports.ui.tweener;
 const ModalDialog = imports.ui.modalDialog;
 const Tooltips = imports.ui.tooltips;
 const PointerTracker = imports.misc.pointerTracker;
+const SignalManager = imports.misc.signalManager;
 const GridNavigator = imports.misc.gridNavigator;
 const WindowUtils = imports.misc.windowUtils;
 
@@ -49,24 +50,12 @@ ExpoWindowClone.prototype = {
         this.realWindow = realWindow;
         this.metaWindow = realWindow.meta_window;
         this.refreshClone();
-        let positionChangedId = this.realWindow.connect('position-changed',
-                                                          Lang.bind(this, this.onPositionChanged));
-        let sizeChangedId = this.realWindow.connect('position-changed',
-                                                          Lang.bind(this, this.onSizeChanged));
-        let orphaned = false;
-        let realWindowDestroyedId = this.realWindow.connect('destroy', Lang.bind(this, function() {
-            orphaned = true;
-        }));
-        let workspaceChangedId = this.metaWindow.connect('workspace-changed', Lang.bind(this, function(w, oldws) {
+        this._signalManager = new SignalManager.SignalManager(null);
+
+        this._signalManager.connect(this.realWindow, 'size-changed', this.onSizeChanged, this);
+        this._signalManager.connect(this.metaWindow, 'workspace-changed', function(w, oldws) {
             this.emit('workspace-changed', oldws);
-        }));
-        this.disconnectWindowSignals = function() {
-            this.metaWindow.disconnect(workspaceChangedId);
-            if (orphaned) return;
-            realWindow.disconnect(sizeChangedId);
-            realWindow.disconnect(positionChangedId);
-            realWindow.disconnect(realWindowDestroyedId);
-        };
+        }, this);
 
         this.onPositionChanged();
         this.onSizeChanged();
@@ -109,7 +98,7 @@ ExpoWindowClone.prototype = {
         this.inDrag = false;
         this.dragCancelled = false;
 
-        this.icon = new St.Group();
+        this.icon = new St.Widget();
         this.actor.add_actor(this.icon);
         this.icon.hide();
 
@@ -123,7 +112,7 @@ ExpoWindowClone.prototype = {
             this.metaWindow._expoApp = app;
         }
         if (app) {
-            iconActor = app.create_icon_texture(ICON_SIZE);
+            iconActor = app.create_icon_texture_for_window(ICON_SIZE, this.metaWindow);
         }
         if (!iconActor) {
             iconActor = new St.Icon({ icon_name: 'applications-other',
@@ -144,7 +133,7 @@ ExpoWindowClone.prototype = {
 
     refreshClone: function(withTransients) {
         if (this.clone) {this.clone.destroy();}
-        this.clone = new St.Group({reactive: false});
+        this.clone = new St.Widget({ reactive: false });
         this.actor.add_actor(this.clone);
         let [pwidth, pheight] = [this.realWindow.width, this.realWindow.height];
         let clones = WindowUtils.createWindowClone(this.metaWindow, 0, 0, withTransients);
@@ -157,10 +146,10 @@ ExpoWindowClone.prototype = {
     },
 
     killUrgencyTimeout: function() {
-        if (this.urgencyTimeout) {
+        if (this.urgencyTimeout != 0) {
             Mainloop.source_remove(this.urgencyTimeout);
+            this.urgencyTimeout = 0;
         }
-        this.urgencyTimeout = 0;
     },
 
     showUrgencyState: function(params) {
@@ -199,6 +188,7 @@ ExpoWindowClone.prototype = {
             this.killUrgencyTimeout();
             this.urgencyTimeout = Mainloop.timeout_add(750, Lang.bind(this, function() {
                 this.showUrgencyState({showUrgent:!force, reps: params.reps - (force ? 0 : 1)});
+                this.urgencyTimeout = 0;
             }));
         }
     },
@@ -242,7 +232,7 @@ ExpoWindowClone.prototype = {
     },
 
     onDestroy: function() {
-        this.disconnectWindowSignals();
+        this._signalManager.disconnectAllSignals();
         this.actor._delegate = null;
 
         if (this.inDrag) {
@@ -296,37 +286,6 @@ const ThumbnailState = {
     DESTROYED :     7
 };
 
-function ConfirmationDialog(prompt, yesAction, yesFocused){
-    this._init(prompt, yesAction, yesFocused);
-}
-
-ConfirmationDialog.prototype = {
-    __proto__: ModalDialog.ModalDialog.prototype,
-
-    _init: function(prompt, yesAction, yesFocused) {
-        ModalDialog.ModalDialog.prototype._init.call(this);
-        let label = new St.Label({text: prompt});
-        this.contentLayout.add(label);
-
-        this.setButtons([
-            {
-                label: _("Yes"),
-                focused: yesFocused,
-                action: Lang.bind(this, function(){
-                    yesAction();
-                    this.close();
-                })
-            },
-            {
-                label: _("No"),
-                action: Lang.bind(this, function(){
-                    this.close();
-                })
-            }
-        ]);
-    },
-};
-
 /**
  * @metaWorkspace: a #Meta.Workspace
  */
@@ -338,11 +297,14 @@ ExpoWorkspaceThumbnail.prototype = {
     _init : function(metaWorkspace, box) {
         this.box = box;
         this.metaWorkspace = metaWorkspace;
-        this.frame = new St.Group({ clip_to_allocation: true,
-                                    style_class: 'expo-workspace-thumbnail-frame' });
-        this.actor = new St.Group({ reactive: true,
-                                    clip_to_allocation: true,
-                                    style_class: 'workspace-thumbnail' });
+
+        this.overviewMode = false;
+
+        this.frame = new St.Widget({ clip_to_allocation: true,
+                                     style_class: 'expo-workspace-thumbnail-frame' });
+        this.actor = new St.Widget({ reactive: true,
+                                     clip_to_allocation: true,
+                                     style_class: 'workspace-thumbnail' });
         this.actor._delegate = this;
         this.actor.set_size(global.screen_width, global.screen_height);
 
@@ -504,9 +466,8 @@ ExpoWorkspaceThumbnail.prototype = {
                 // Use the stable sequence for an integer to use as a hash key
                 this.stackIndices[stack[i].get_meta_window().get_stable_sequence()] = i;
             }
+            this.syncStacking(this.stackIndices);
         }
-
-        this.syncStacking(this.stackIndices);
     },
 
     setActive: function(isActive) {
@@ -887,6 +848,9 @@ ExpoWorkspaceThumbnail.prototype = {
     },
 
     onScrollEvent: function (actor, event) {
+        if (Main.expo.animationInProgress)
+            return;
+
         switch ( event.get_scroll_direction() ) {
         case Clutter.ScrollDirection.UP:
             Main.wm.actionMoveWorkspaceLeft();
@@ -935,7 +899,7 @@ ExpoWorkspaceThumbnail.prototype = {
             this.highlight();
             let prompt = _("Are you sure you want to remove workspace \"%s\"?\n\n").format(
                 Main.getWorkspaceName(this.metaWorkspace.index()));
-            let confirm = new ConfirmationDialog(prompt, removeAction, true);
+            let confirm = new ModalDialog.ConfirmDialog(prompt, removeAction);
             confirm.open();
         }
         else {
@@ -996,7 +960,7 @@ ExpoWorkspaceThumbnail.prototype = {
             if (dropping) {
                 metaWindow.move_to_monitor(targetMonitor);
                 if (targetMonitor === Main.layoutManager.primaryIndex) {
-                    metaWindow.change_workspace(this.metaWorkspace, false, time);
+                    metaWindow.change_workspace(this.metaWorkspace);
                 }
             }
         }
@@ -1010,7 +974,7 @@ ExpoWorkspaceThumbnail.prototype = {
             {
                 canDrop = true;
                 if (dropping) {
-                    metaWindow.change_workspace(this.metaWorkspace, false, time);
+                    metaWindow.change_workspace(this.metaWorkspace);
                 }
             }
             if (movingMonitors) {
@@ -1284,7 +1248,9 @@ ExpoThumbnailsBox.prototype = {
                 }
             };
             thumbnail.actor.connect('destroy', Lang.bind(this, function(actor) {
-                setOverviewTimeout(0, null);
+                setOverviewTimeout(0, function() {
+                    overviewTimeoutId = 0;
+                });
                 this.actor.remove_actor(thumbnail.frame);
                 this.actor.remove_actor(actor);
                 this.actor.remove_actor(thumbnail.title);
@@ -1322,6 +1288,7 @@ ExpoThumbnailsBox.prototype = {
                         if (thumbnail.hovering) {
                             thumbnail.overviewModeOn();
                         }
+                        overviewTimeoutId = 0;
                     });
                 }
             }));
@@ -1337,6 +1304,7 @@ ExpoThumbnailsBox.prototype = {
                         if (!thumbnail.hovering) {
                             thumbnail.overviewModeOff();
                         }
+                        overviewTimeoutId = 0;
                     });
                 }
             }));
@@ -1692,8 +1660,8 @@ ExpoThumbnailsBox.prototype = {
             x += thumbnailWidth + spacing;
             y += (i + 1) % nColumns > 0 ? 0 : thumbnailHeight + extraHeight + thTitleMargin;
         }
-        let x = 0;
-        let y = 0;
+        x = 0;
+        y = 0;
 
         let buttonWidth = this.button.get_theme_node().get_length('width');
         let buttonHeight = this.button.get_theme_node().get_length('height');
@@ -1737,6 +1705,9 @@ ExpoThumbnailsBox.prototype = {
     },
 
     onScrollEvent: function (actor, event) {
+        if (Main.expo.animationInProgress)
+            return;
+
         switch ( event.get_scroll_direction() ) {
         case Clutter.ScrollDirection.UP:
             Main.wm.actionMoveWorkspaceUp();
