@@ -29,7 +29,14 @@ const ScreenPosition = {
     RIGHT_HALF: 5
 };
 
-const MOUSE_POLL_FREQUENCY = 50;
+const LensShape = {
+    NONE: 0,
+    SQUARE : 1,
+    HORIZONTAL : 2,
+    VERTICAL : 3
+}
+
+const MOUSE_POLL_FREQUENCY = 15;
 const CROSSHAIRS_CLIP_SIZE = [100, 100];
 
 // Settings
@@ -40,6 +47,7 @@ const MAGNIFIER_SCHEMA          = 'org.cinnamon.desktop.a11y.magnifier';
 const SCREEN_POSITION_KEY       = 'screen-position';
 const MAG_FACTOR_KEY            = 'mag-factor';
 const LENS_MODE_KEY             = 'lens-mode';
+const LENS_SHAPE_KEY            = 'lens-shape';
 const CLAMP_MODE_KEY            = 'scroll-at-edges';
 const MOUSE_TRACKING_KEY        = 'mouse-tracking';
 const SHOW_CROSS_HAIRS_KEY      = 'show-cross-hairs';
@@ -49,7 +57,7 @@ const CROSS_HAIRS_OPACITY_KEY   = 'cross-hairs-opacity';
 const CROSS_HAIRS_LENGTH_KEY    = 'cross-hairs-length';
 const CROSS_HAIRS_CLIP_KEY      = 'cross-hairs-clip';
 
-const MEDIA_KEYS_SCHEMA         = "org.cinnamon.settings-daemon.plugins.media-keys"
+const KEYBINDING_SCHEMA         = "org.cinnamon.desktop.keybindings"
 const ZOOM_IN_KEY               = "magnifier-zoom-in"
 const ZOOM_OUT_KEY              = "magnifier-zoom-out"
 
@@ -65,11 +73,42 @@ Magnifier.prototype = {
         // Magnifier is a manager of ZoomRegions.
         this._zoomRegions = [];
 
-        this.enabled = false;
+        this._appSettings = new Gio.Settings({ schema_id: APPLICATIONS_SCHEMA });
+        this._settings = new Gio.Settings({ schema_id: MAGNIFIER_SCHEMA });
 
-        this._appSettings = new Gio.Settings({ schema: APPLICATIONS_SCHEMA });
-        this._settings = new Gio.Settings({ schema: MAGNIFIER_SCHEMA });
+        this._initialized = false;
+        this.update_mag_id = 0;
+        this.enabled = this._appSettings.get_boolean(SHOW_KEY);
 
+        this._appSettings.connect('changed::' + SHOW_KEY,
+            () => {
+                this.enabled = this._appSettings.get_boolean(SHOW_KEY);
+                let factor = parseFloat(this._settings.get_double(MAG_FACTOR_KEY).toFixed(2));
+                if (this.enabled) {
+                    if (factor > 1.0)
+                        this.setActive(true);
+                    else
+                        this._initialize();
+                } else {
+                    if (this.isActive())
+                        this.setActive(false);
+                }
+            });
+
+        // Export to dbus.
+        magDBusService = new MagnifierDBus.CinnamonMagnifier(this.enabled);
+        magInputHandler = new MagnifierInputHandler(this);
+
+        let factor = parseFloat(this._settings.get_double(MAG_FACTOR_KEY).toFixed(2));
+        if (this.enabled && factor > 1.0)
+            this.setActive(true);
+    },
+
+    _initialize: function() {
+        if (this._initialized)
+            return;
+
+        this._initialized = true;
         // Create small clutter tree for the magnified mouse.
         let xfixesCursor = Cinnamon.XFixesCursor.get_for_stage(global.stage);
         this._mouseSprite = new Clutter.Texture();
@@ -77,37 +116,17 @@ Magnifier.prototype = {
         this._cursorRoot = new Clutter.Group();
         this._cursorRoot.add_actor(this._mouseSprite);
 
-        // Create the first ZoomRegion and initialize it according to the
-        // magnification settings.
-
-        let mask;
-        [this.xMouse, this.yMouse, mask] = global.get_pointer();
-
-        let aZoomRegion = new ZoomRegion(this, this._cursorRoot);
-        this._zoomRegions.push(aZoomRegion);
-        let activeAtLaunch = this._settingsInit(aZoomRegion);
-        aZoomRegion.scrollContentsTo(this.xMouse, this.yMouse);
+        [this.xMouse, this.yMouse, ] = global.get_pointer();
 
         xfixesCursor.connect('cursor-change', Lang.bind(this, this._updateMouseSprite));
         this._xfixesCursor = xfixesCursor;
 
-        this._appSettings.connect('changed::' + SHOW_KEY,
-                                  Lang.bind(this, function() {
-            this.enabled = this._appSettings.get_boolean(SHOW_KEY);
-            let factor = parseFloat(this._settings.get_double(MAG_FACTOR_KEY).toFixed(2));
-            this.setActive(this.enabled && factor > 1.0);
-        }));
-
-        this.enabled = this._appSettings.get_boolean(SHOW_KEY);
-
-        // Export to dbus.
-        magDBusService = new MagnifierDBus.CinnamonMagnifier(this.enabled);
-
-        this.setActive(this.enabled && activeAtLaunch);
-
-        magInputHandler = new MagnifierInputHandler(this);
-
-        this.update_mag_id = 0;
+        // Create the first ZoomRegion and initialize it according to the
+        // magnification settings.
+        let aZoomRegion = new ZoomRegion(this, this._cursorRoot);
+        this._zoomRegions.push(aZoomRegion);
+        aZoomRegion.scrollContentsTo(this.xMouse, this.yMouse);
+        this._settingsInit(aZoomRegion);
     },
 
     /**
@@ -115,6 +134,7 @@ Magnifier.prototype = {
      * Show the system mouse pointer.
      */
     showSystemCursor: function() {
+        this._initialize();
         this._xfixesCursor.show();
     },
 
@@ -123,6 +143,7 @@ Magnifier.prototype = {
      * Hide the system mouse pointer.
      */
     hideSystemCursor: function() {
+        this._initialize();
         this._xfixesCursor.hide();
     },
 
@@ -132,6 +153,11 @@ Magnifier.prototype = {
      * @activate:   Boolean to activate or de-activate the magnifier.
      */
     setActive: function(activate) {
+        if (!activate && !this._initialized)
+            return;
+
+        this._initialize();
+
         this._zoomRegions.forEach (function(zoomRegion, index, array) {
             zoomRegion.setActive(activate);
         });
@@ -152,6 +178,7 @@ Magnifier.prototype = {
 
     _write_back_mag_factor: function(factor) {
         this._settings.set_double(MAG_FACTOR_KEY, factor);
+        this.update_mag_id = 0;
         return false;
     },
 
@@ -164,6 +191,8 @@ Magnifier.prototype = {
      *                  of the magnified view.
      */
     setMagFactor: function(xMagFactor, yMagFactor) {
+        this._initialize();
+
         let zr = this.getZoomRegions()[0];
         zr.setMagFactor(xMagFactor, yMagFactor);
 
@@ -192,6 +221,7 @@ Magnifier.prototype = {
      * Turn on mouse tracking, if not already doing so.
      */
     startTrackingMouse: function() {
+        this._initialize();
         if (!this._mouseTrackingId)
             this._mouseTrackingId = Mainloop.timeout_add(
                 MOUSE_POLL_FREQUENCY,
@@ -225,6 +255,8 @@ Magnifier.prototype = {
      * @return      true.
      */
     scrollToMousePos: function() {
+        this._initialize();
+
         let [xMouse, yMouse, mask] = global.get_pointer();
 
         if (xMouse != this.xMouse || yMouse != this.yMouse) {
@@ -260,11 +292,13 @@ Magnifier.prototype = {
      * @return          The newly created ZoomRegion.
      */
     createZoomRegion: function(xMagFactor, yMagFactor, roi, viewPort) {
+        this._initialize();
+
         let zoomRegion = new ZoomRegion(this, this._cursorRoot);
         zoomRegion.setViewPort(viewPort);
 
         // We ignore the redundant width/height on the ROI
-        let fixedROI = new Object(roi);
+        let fixedROI = Object.assign({}, roi);
         fixedROI.width = viewPort.width / xMagFactor;
         fixedROI.height = viewPort.height / yMagFactor;
         zoomRegion.setROI(fixedROI);
@@ -280,6 +314,8 @@ Magnifier.prototype = {
      * @zoomRegion:     The zoomRegion to add.
      */
     addZoomRegion: function(zoomRegion) {
+        this._initialize();
+
         if(zoomRegion) {
             this._zoomRegions.push(zoomRegion);
             if (!this.isTrackingMouse())
@@ -301,6 +337,9 @@ Magnifier.prototype = {
      * Remove all the zoom regions from this Magnfier's ZoomRegion list.
      */
     clearAllZoomRegions: function() {
+        if (!this._initialized)
+            return;
+
         for (let i = 0; i < this._zoomRegions.length; i++)
             this._zoomRegions[i].setActive(false);
 
@@ -314,6 +353,8 @@ Magnifier.prototype = {
      * Add and show a cross hair centered on the magnified mouse.
      */
     addCrosshairs: function() {
+        this._initialize();
+
         if (!this._crossHairs)
             this._crossHairs = new Crosshairs();
 
@@ -359,9 +400,9 @@ Magnifier.prototype = {
      */
     setCrosshairsColor: function(color) {
         if (this._crossHairs) {
-            let clutterColor = new Clutter.Color();
-            clutterColor.from_string(color);
-            this._crossHairs.setColor(clutterColor);
+            let [success, cc] = Clutter.Color.from_string(color);
+            if (success)
+                this._crossHairs.setColor(cc);
         }
     },
 
@@ -470,14 +511,14 @@ Magnifier.prototype = {
      * Get whether the crosshairs are clipped by the mouse image.
      * @return:   Whether the crosshairs are clipped.
      */
-     getCrosshairsClip: function() {
+    getCrosshairsClip: function() {
         if (this._crossHairs) {
             let [clipWidth, clipHeight] = this._crossHairs.getClip();
             return (clipWidth > 0 && clipHeight > 0);
         }
         else
             return false;
-     },
+    },
 
     //// Private methods ////
 
@@ -488,6 +529,16 @@ Magnifier.prototype = {
         this._mouseSprite.set_anchor_point(xHot, yHot);
     },
 
+    _updateZoomRegion: function(zoomRegion) {
+        if (zoomRegion._lensMode) {
+            let pref = this._settings.get_enum(LENS_SHAPE_KEY);
+            zoomRegion.setLensShape(pref);
+        } else {
+            let pref = this._settings.get_enum(SCREEN_POSITION_KEY);
+            zoomRegion.setScreenPosition(pref);
+        }
+    },
+
     _settingsInit: function(zoomRegion) {
         let ret = 1.0;
         if (zoomRegion) {
@@ -497,13 +548,11 @@ Magnifier.prototype = {
             if (aPref > 1.0)
                 zoomRegion.setMagFactor(aPref, aPref);
 
-            aPref = this._settings.get_enum(SCREEN_POSITION_KEY);
-            if (aPref)
-                zoomRegion.setScreenPosition(aPref);
-
             zoomRegion.setLensMode(this._settings.get_boolean(LENS_MODE_KEY));
             zoomRegion.setClampScrollingAtEdges(!this._settings.get_boolean(CLAMP_MODE_KEY));
 
+            this._updateZoomRegion(zoomRegion)
+
             aPref = this._settings.get_enum(MOUSE_TRACKING_KEY);
             if (aPref)
                 zoomRegion.setMouseTrackingMode(aPref);
@@ -515,10 +564,12 @@ Magnifier.prototype = {
 
         this._settings.connect('changed::' + SCREEN_POSITION_KEY,
                                Lang.bind(this, this._updateScreenPosition));
+        this._settings.connect('changed::' + LENS_SHAPE_KEY,
+                               Lang.bind(this, this._updateLensShape));
         this._settings.connect('changed::' + MAG_FACTOR_KEY,
                                Lang.bind(this, this._updateMagFactor));
         this._settings.connect('changed::' + LENS_MODE_KEY,
-                               Lang.bind(this, this._updateLensMode));
+                               Lang.bind(this, this._updateLensShape));
         this._settings.connect('changed::' + CLAMP_MODE_KEY,
                                Lang.bind(this, this._updateClampMode));
         this._settings.connect('changed::' + MOUSE_TRACKING_KEY,
@@ -555,7 +606,7 @@ Magnifier.prototype = {
         }));
 
         return ret > 1.0;
-   },
+    },
 
     _updateScreenPosition: function() {
         // Applies only to the first zoom region.
@@ -567,6 +618,15 @@ Magnifier.prototype = {
         }
     },
 
+    _updateLensShape: function() {
+        // Applies only to the first zoom region.
+        if (this._zoomRegions.length) {
+            let shape = this._settings.get_enum(LENS_SHAPE_KEY);
+            this._zoomRegions[0].setLensShape(shape);
+            this._updateLensMode();
+        }
+    },
+
     _updateMagFactor: function() {
         // Applies only to the first zoom region.
         if (this._zoomRegions.length) {
@@ -616,7 +676,7 @@ ZoomRegion.prototype = {
         this._clampScrollingAtEdges = false;
         this._lensMode = false;
         this._screenPosition = ScreenPosition.FULL_SCREEN;
-
+        this._lensShape = LensShape.NONE;
         this._magView = null;
         this._uiGroupClone = null;
         this._mouseSourceActor = mouseSourceActor;
@@ -759,8 +819,10 @@ ZoomRegion.prototype = {
      */
     setLensMode: function(lensMode) {
         this._lensMode = lensMode;
-        if (!this._lensMode)
-            this.setScreenPosition (this._screenPosition);
+        if (this._lensMode)
+            this.setLensShape(this._lensShape);
+        else
+            this.setScreenPosition(this._screenPosition);
     },
 
     /**
@@ -784,6 +846,48 @@ ZoomRegion.prototype = {
             this._changeROI();
     },
 
+    /**
+     * setSquareLens:
+     * Magnifier view occupies a square on the screen.
+     */
+    setSquareLens: function() {
+        let viewPort = {};
+        viewPort.x = 0;
+        viewPort.y = 0;
+        viewPort.height = global.screen_height / 2;
+        viewPort.width = global.screen_height / 2; /* Keep it square */
+        this._setViewPort(viewPort);
+        this._lensShape = LensShape.SQUARE;
+    },
+
+    /**
+     * setHorizontalLens:
+     * Magnifier view occupies the top half of the screen.
+     */
+    setHorizontalLens: function() {
+        let viewPort = {};
+        viewPort.x = 0;
+        viewPort.y = 0;
+        viewPort.width = global.screen_width;
+        viewPort.height = global.screen_height/2;
+        this._setViewPort(viewPort);
+        this._lensShape = LensShape.HORIZONTAL;
+    },
+
+    /**
+     * setVerticalLens:
+     * Magnifier view occupies the left half of the screen.
+     */
+    setVerticalLens: function() {
+        let viewPort = {};
+        viewPort.x = 0;
+        viewPort.y = 0;
+        viewPort.width = global.screen_width/2;
+        viewPort.height = global.screen_height;
+        this._setViewPort(viewPort);
+        this._lensShape = LensShape.VERTICAL;
+    },
+
     /**
      * setTopHalf:
      * Magnifier view occupies the top half of the screen.
@@ -860,12 +964,12 @@ ZoomRegion.prototype = {
      * setScreenPosition:
      * Positions the zoom region to one of the enumerated positions on the
      * screen.
-     * @position:   one of Magnifier.FULL_SCREEN, Magnifier.TOP_HALF,
-     *              Magnifier.BOTTOM_HALF,Magnifier.LEFT_HALF, or
-     *              Magnifier.RIGHT_HALF.
+     * @position:   one of ScreenPosition.FULL_SCREEN, ScreenPosition.TOP_HALF,
+     *              ScreenPosition.BOTTOM_HALF,ScreenPosition.LEFT_HALF, or
+     *              ScreenPosition.RIGHT_HALF.
      */
-    setScreenPosition: function(inPosition) {
-        switch (inPosition) {
+    setScreenPosition: function(position) {
+        switch (position) {
             case ScreenPosition.FULL_SCREEN:
                 this.setFullScreenMode();
                 break;
@@ -884,6 +988,25 @@ ZoomRegion.prototype = {
         }
     },
 
+    /**
+     * setLensShape:
+     * Sets the shape of the zoom lens
+     * @shape:      LensShape.SQUARE, LensShape.HORIZONTAL, LensShape.VERTICAL.
+     */
+    setLensShape: function(shape) {
+        switch (shape) {
+            case LensShape.SQUARE:
+                this.setSquareLens();
+                break;
+            case LensShape.HORIZONTAL:
+                this.setHorizontalLens();
+                break;
+            case LensShape.VERTICAL:
+                this.setVerticalLens();
+                break;
+        }
+    },
+
     /**
      * getScreenPosition:
      * Tell the outside world what the current mode is -- magnifiying the
@@ -894,6 +1017,16 @@ ZoomRegion.prototype = {
         return this._screenPosition;
     },
 
+    /**
+     * getLensShape:
+     * Get the shape of the zoom lens
+     *
+     * @return:  the current shape.
+     */
+    getLensShape: function() {
+        return this._lensShape;
+    },
+
     /**
      * scrollToMousePos:
      * Set the region of interest based on the position of the system pointer.
@@ -1475,6 +1608,7 @@ Crosshairs.prototype = {
 };
 
 const INCR = 0.1;
+const MAX_ZOOM = 15.0; /* from range of org.cinnamon.desktop.a11y.magnifier mag-factor key */
 
 function MagnifierInputHandler(magnifier) {
     this._init(magnifier);
@@ -1487,11 +1621,11 @@ MagnifierInputHandler.prototype = {
         this._zoom_in_id = 0;
         this._zoom_out_id = 0;
 
-        this.a11y_settings = new Gio.Settings({ schema: APPLICATIONS_SCHEMA });
+        this.a11y_settings = new Gio.Settings({ schema_id: APPLICATIONS_SCHEMA });
         this.a11y_settings.connect("changed::" + SHOW_KEY, Lang.bind(this, this._refresh_state));
 
-        this.media_keys_settings = new Gio.Settings({ schema: MEDIA_KEYS_SCHEMA });
-        this.media_keys_settings.connect("changed", Lang.bind(this, this._refresh_state));
+        this.keybinding_settings = new Gio.Settings({ schema_id: KEYBINDING_SCHEMA });
+        this.keybinding_settings.connect("changed", Lang.bind(this, this._refresh_state));
 
         this._refresh_state();
     },
@@ -1502,10 +1636,10 @@ MagnifierInputHandler.prototype = {
         this._zoom_in_id = global.display.connect('zoom-scroll-in', Lang.bind(this, this._zoom_in));
         this._zoom_out_id = global.display.connect('zoom-scroll-out', Lang.bind(this, this._zoom_out));
 
-        let kb = this.media_keys_settings.get_string(ZOOM_IN_KEY);
-        Main.keybindingManager.addHotKey("magnifier-zoom-in", kb, Lang.bind(this, this._zoom_in));
-        kb = this.media_keys_settings.get_string(ZOOM_OUT_KEY);
-        Main.keybindingManager.addHotKey("magnifier-zoom-out", kb, Lang.bind(this, this._zoom_out));
+        let kb = this.keybinding_settings.get_strv(ZOOM_IN_KEY);
+        Main.keybindingManager.addHotKeyArray("magnifier-zoom-in", kb, Lang.bind(this, this._zoom_in));
+        kb = this.keybinding_settings.get_strv(ZOOM_OUT_KEY);
+        Main.keybindingManager.addHotKeyArray("magnifier-zoom-out", kb, Lang.bind(this, this._zoom_out));
     },
 
     _disable_zoom: function() {
@@ -1538,9 +1672,9 @@ MagnifierInputHandler.prototype = {
 
     _zoom_in: function(display, screen, event, kb, action) {
         if (this.zoom_active) {
-            this.current_zoom *= (1.0 + INCR);
+            this.current_zoom = Math.min(this.current_zoom * (1.0 + INCR), MAX_ZOOM);
         } else {
-            this.current_zoom *= (1.0 + INCR);
+            this.current_zoom *= Math.min(this.current_zoom * (1.0 + INCR), MAX_ZOOM);
             this.magnifier.setActive(true)
             this.zoom_active = true;
         }
