@@ -4,6 +4,7 @@ const Clutter = imports.gi.Clutter;
 const GLib = imports.gi.GLib;
 const Gio = imports.gi.Gio;
 const Gtk = imports.gi.Gtk;
+const Atk = imports.gi.Atk;
 const Lang = imports.lang;
 const Mainloop = imports.mainloop;
 const Meta = imports.gi.Meta;
@@ -20,23 +21,22 @@ const Tweener = imports.ui.tweener;
 const Util = imports.misc.util;
 const AppletManager = imports.ui.appletManager;
 
-const ANIMATION_TIME = .2;
-const NOTIFICATION_TIMEOUT = 4;
-const NOTIFICATION_CRITICAL_TIMEOUT_WITH_APPLET = 10;
-const SUMMARY_TIMEOUT = 1;
-const LONGER_SUMMARY_TIMEOUT = 4;
+var ANIMATION_TIME = .2;
+var NOTIFICATION_TIMEOUT = 4;
+var NOTIFICATION_CRITICAL_TIMEOUT_WITH_APPLET = 10;
+var SUMMARY_TIMEOUT = 1;
+var LONGER_SUMMARY_TIMEOUT = 4;
 
-const HIDE_TIMEOUT = 0.2;
-const LONGER_HIDE_TIMEOUT = 0.6;
-
-const MAX_SOURCE_TITLE_WIDTH = 180;
+var HIDE_TIMEOUT = 0.2;
+var LONGER_HIDE_TIMEOUT = 0.6;
 
+var MAX_SOURCE_TITLE_WIDTH = 180;
 
 // We delay hiding of the tray if the mouse is within MOUSE_LEFT_ACTOR_THRESHOLD
 // range from the point where it left the tray.
-const MOUSE_LEFT_ACTOR_THRESHOLD = 20;
+var MOUSE_LEFT_ACTOR_THRESHOLD = 20;
 
-const State = {
+var State = {
     HIDDEN:  0,
     SHOWING: 1,
     SHOWN:   2,
@@ -48,7 +48,7 @@ const State = {
 // user did not interact with, DISMISSED for all other notifications that were
 // destroyed as a result of a user action, and SOURCE_CLOSED for the notifications
 // that were requested to be destroyed by the associated source.
-const NotificationDestroyedReason = {
+var NotificationDestroyedReason = {
     EXPIRED: 1,
     DISMISSED: 2,
     SOURCE_CLOSED: 3
@@ -58,7 +58,7 @@ const NotificationDestroyedReason = {
 // urgency values map to the corresponding values for the notifications received
 // through the notification daemon. HIGH urgency value is used for chats received
 // through the Telepathy client.
-const Urgency = {
+var Urgency = {
     LOW: 0,
     NORMAL: 1,
     HIGH: 2,
@@ -195,21 +195,21 @@ URLHighlighter.prototype = {
     },
 
     _findUrlAtPos: function(event) {
+        if (!this._urls.length)
+            return -1;
+
         let success;
         let [x, y] = event.get_coords();
-        [success, x, y] = this.actor.transform_stage_point(x, y);
-        let find_pos = -1;
-        for (let i = 0; i < this.actor.clutter_text.text.length; i++) {
-            let [success, px, py, line_height] = this.actor.clutter_text.position_to_coords(i);
-            if (py > y || py + line_height < y || x < px)
-                continue;
-            find_pos = i;
-        }
-        if (find_pos != -1) {
-            for (let i = 0; i < this._urls.length; i++)
-            if (find_pos >= this._urls[i].pos &&
-                this._urls[i].pos + this._urls[i].url.length > find_pos)
-                return i;
+        let ct = this.actor.clutter_text;
+        [success, x, y] = ct.transform_stage_point(x, y);
+        if (success && x >= 0 && x <= ct.width
+                    && y >= 0 && y <= ct.height) {
+            let pos = ct.coords_to_position(x, y);
+            for (let i = 0; i < this._urls.length; i++) {
+                let url = this._urls[i]
+                if (pos >= url.pos && pos <= url.pos + url.url.length)
+                    return i;
+            }
         }
         return -1;
     }
@@ -433,6 +433,7 @@ Notification.prototype = {
         // 'transient' is a reserved keyword in JS, so we have to use an alternate variable name
         this.isTransient = false;
         this.expanded = false;
+        this.silent = false;
         this._destroyed = false;
         this._useActionIcons = false;
         this._customContent = false;
@@ -445,15 +446,13 @@ Notification.prototype = {
         this._imageBin = null;
         this._timestamp = new Date();
         this._inNotificationBin = false;
-        this.dateFormat = _("%l:%M %p");
 
         source.connect('destroy', Lang.bind(this,
             function (source, reason) {
                 this.destroy(reason);
             }));
 
-        this.actor = new St.Button();
-        this.actor._delegate = this;
+        this.actor = new St.Button({ accessible_role: Atk.Role.NOTIFICATION });
         this.actor._parent_container = null;
         this.actor.connect('clicked', Lang.bind(this, this._onClicked));
         this.actor.connect('destroy', Lang.bind(this, this._onDestroy));
@@ -488,6 +487,21 @@ Notification.prototype = {
                                         col: 2,
                                         y_expand: false,
                                         y_fill: false });
+
+        // notification dismiss button
+        let icon = new St.Icon({ icon_name: 'window-close',
+                                 icon_type: St.IconType.SYMBOLIC,
+                                 icon_size: 16 });
+        let closeButton = new St.Button({ child: icon, opacity: 128 });
+        closeButton.connect('clicked', Lang.bind(this, this.destroy));
+        closeButton.connect('notify::hover', function() { closeButton.opacity = closeButton.hover ? 255 : 128; });
+        this._table.add(closeButton, { row: 0,
+                                       col: 3,
+                                       x_expand: false,
+                                       y_expand: false,
+                                       y_fill: false,
+                                       y_align: St.Align.START });
+
         this._timeLabel = new St.Label();
         this._titleLabel = new St.Label();
         this._bannerBox.add_actor(this._titleLabel);
@@ -517,9 +531,11 @@ Notification.prototype = {
                                         titleMarkup: false,
                                         bannerMarkup: false,
                                         bodyMarkup: false,
+                                        silent: false,
                                         clear: false });
 
         this._customContent = params.customContent;
+        this.silent = params.silent;
 
         let oldFocus = global.stage.key_focus;
 
@@ -566,7 +582,7 @@ Notification.prototype = {
         this.title = title;
         title = title ? _fixMarkup(title.replace(/\n/g, ' '), params.titleMarkup) : '';
         this._titleLabel.clutter_text.set_markup('<b>' + title + '</b>');
-        this._timeLabel.clutter_text.set_markup(this._timestamp.toLocaleTimeString(this.dateFormat));
+        this._timeLabel.clutter_text.set_markup(this._timestamp.toLocaleTimeString());
         this._timeLabel.hide();
         if (Pango.find_base_dir(title, -1) == Pango.Direction.RTL)
             this._titleDirection = St.TextDirection.RTL;
@@ -608,10 +624,22 @@ Notification.prototype = {
 
     _createScrollArea: function() {
         this._table.add_style_class_name('multi-line-notification');
+
+        // FIXME: this doesn't actually scroll/limit notification size with the current policies.
+        // if we allow scrolling, then there doesn't seem to be a minimum height when inside the
+        // tray which breaks the layout and in the extreme case makes the notifications unreadable
         this._scrollArea = new St.ScrollView({ name: 'notification-scrollview',
                                                vscrollbar_policy: Gtk.PolicyType.NEVER,
                                                hscrollbar_policy: Gtk.PolicyType.NEVER,
                                                style_class: 'vfade' });
+
+        // prevent non-scrollable notifications from taking scroll events, otherwise we can't
+        // easily scroll the message tray.
+        // FIXME: if we enable scrolling then we may want to toggle this based on vscrollbar_visible
+        // or whether the notification is in the tray.
+        // something like: scrollArea.connect("notify::vscrollbar-visible", () => (enable = visible));
+        this._scrollArea.enable_mouse_scrolling = false;
+
         this._table.add(this._scrollArea, { row: 1,
                                             col: 2 });
         this._updateLastColumnSettings();
@@ -702,10 +730,10 @@ Notification.prototype = {
     _updateLastColumnSettings: function() {
         if (this._scrollArea)
             this._table.child_set(this._scrollArea, { col: this._imageBin ? 2 : 1,
-                                                      col_span: this._imageBin ? 1 : 2 });
+                                                      col_span: this._imageBin ? 2 : 3 });
         if (this._actionArea)
             this._table.child_set(this._actionArea, { col: this._imageBin ? 2 : 1,
-                                                      col_span: this._imageBin ? 1 : 2 });
+                                                      col_span: this._imageBin ? 2 : 3 });
     },
 
     setImage: function(image) {
@@ -770,7 +798,7 @@ Notification.prototype = {
             button.label = label;
         }
 
-        if (this._buttonBox.get_children().length > 0)
+        if (this._buttonBox.get_n_children() > 0)
             this._buttonFocusManager.remove_group(this._buttonBox);
 
         this._buttonBox.add(button);
@@ -830,10 +858,10 @@ Notification.prototype = {
 
         let [titleMinW, titleNatW] = this._titleLabel.get_preferred_width(-1);
         let [titleMinH, titleNatH] = this._titleLabel.get_preferred_height(availWidth);
-        
+
         let [timeMinW, timeNatW] = this._timeLabel.get_preferred_width(-1);
         let [timeMinH, timeNatH] = this._timeLabel.get_preferred_height(availWidth);
-        
+
         let [bannerMinW, bannerNatW] = this._bannerLabel.get_preferred_width(availWidth);
 
         let titleBox = new Clutter.ActorBox();
@@ -868,7 +896,7 @@ Notification.prototype = {
         this._titleFitsInBannerMode = (titleNatW <= availWidth);
 
         let bannerFits = true;
-        
+
         if (titleBoxW + this._spacing > availWidth) {
             this._bannerLabel.opacity = 0;
             bannerFits = false;
@@ -1004,7 +1032,6 @@ Notification.prototype = {
     destroy: function(reason) {
         this._destroyedReason = reason;
         this.actor.destroy();
-        this.actor._delegate = null;
     }
 };
 Signals.addSignalMethods(Notification.prototype);
@@ -1015,6 +1042,7 @@ function Source(title) {
 
 Source.prototype = {
     ICON_SIZE: 24,
+    MAX_NOTIFICATIONS: 10,
 
     _init: function(title) {
         this.title = title;
@@ -1094,6 +1122,10 @@ Source.prototype = {
 
     _updateCount: function() {
         let count = this.notifications.length;
+        if (count > this.MAX_NOTIFICATIONS) {
+            let oldestNotif = this.notifications.shift();
+            oldestNotif.destroy();
+        }
         this._setCount(count, count > 1);
     },
 
@@ -1289,7 +1321,7 @@ SummaryItem.prototype = {
     },
 
     prepareNotificationStackForShowing: function() {
-        if (this.notificationStack.get_children().length > 0)
+        if (this.notificationStack.get_n_children() > 0)
             return;
 
         for (let i = 0; i < this.source.notifications.length; i++) {
@@ -1298,7 +1330,6 @@ SummaryItem.prototype = {
     },
 
     doneShowingNotificationStack: function() {
-        let notificationActors = this.notificationStack.get_children();
         for (let i = 0; i < this._stackedNotifications.length; i++) {
             let stackedNotification = this._stackedNotifications[i];
             let notification = stackedNotification.notification;
@@ -1325,7 +1356,7 @@ SummaryItem.prototype = {
         stackedNotification.notificationDoneDisplayingId = notification.connect('done-displaying', Lang.bind(this, this._notificationDoneDisplaying));
         stackedNotification.notificationDestroyedId = notification.connect('destroy', Lang.bind(this, this._notificationDestroyed));
         this._stackedNotifications.push(stackedNotification);
-        if (this.notificationStack.get_children().length > 0)
+        if (this.notificationStack.get_n_children() > 0)
             notification.setIconVisible(false);
         this.notificationStack.add(notification.actor);
         notification.expand(false);
@@ -1364,8 +1395,8 @@ SummaryItem.prototype = {
             }
         }
 
-        if (this.notificationStack.get_children().length > 0)
-            this.notificationStack.get_children()[0]._delegate.setIconVisible(true);
+        if (this.notificationStack.get_n_children() > 0)
+            this.notificationStack.get_child_at_index(0)._delegate.setIconVisible(true);
     }
 };
 Signals.addSignalMethods(SummaryItem.prototype);
@@ -1376,19 +1407,24 @@ function MessageTray() {
 
 MessageTray.prototype = {
     _init: function() {
-        this._presence = new GnomeSession.Presence();
+        this._presence = new GnomeSession.Presence(Lang.bind(this, function(proxy, error) {
+            this._onStatusChanged(proxy.status);
+        }));
+
         this._userStatus = GnomeSession.PresenceStatus.AVAILABLE;
         this._busy = false;
         this._backFromAway = false;
-        this._presence.connect('StatusChanged', Lang.bind(this, this._onStatusChanged));
-        this._presence.getStatus(Lang.bind(this, this._onStatusChanged));
+
+        this._presence.connectSignal('StatusChanged', Lang.bind(this, function(proxy, senderName, [status]) {
+            this._onStatusChanged(status);
+        }));
 
         this._notificationBin = new St.Bin();
         this._notificationBin.hide();
         this._notificationQueue = [];
         this._notification = null;
         this._notificationClickedId = 0;
-        
+
         this._pointerBarrier = 0;
 
         this._focusGrabber = new FocusGrabber();
@@ -1404,25 +1440,29 @@ MessageTray.prototype = {
         this._traySummoned = false;
         this._useLongerTrayLeftTimeout = false;
         this._trayLeftTimeoutId = 0;
-        this._pointerInTray = false;
-        this._pointerInKeyboard = false;
         this._notificationState = State.HIDDEN;
         this._notificationTimeoutId = 0;
         this._notificationExpandedId = 0;
         this._notificationRemoved = false;
         this._reNotifyAfterHideNotification = null;
-        
+
         this._sources = [];
         Main.layoutManager.addChrome(this._notificationBin);
 
         Main.layoutManager.connect('monitors-changed', Lang.bind(this, this._setSizePosition));
 
-        let onNotificationEnabledUpdated = Lang.bind(this, function() {
-            this._notificationsEnabled = global.settings.get_boolean("display-notifications");
+		// Settings
+        this.settings = new Gio.Settings({ schema_id: "org.cinnamon.desktop.notifications" })
+		function setting(self, source, camelCase, dashed) {
+			function updater() { self[camelCase] = source.get_boolean(dashed); }
+			source.connect('changed::'+dashed, updater);
+			updater();
+		}
+		setting(this, this.settings, "_notificationsEnabled", "display-notifications");
+        this.bottomPosition = this.settings.get_boolean("bottom-notifications");
+        this.settings.connect("changed::bottom-notifications", () => {
+            this.bottomPosition = this.settings.get_boolean("bottom-notifications");
         });
-        global.settings.connect('changed::display-notifications', onNotificationEnabledUpdated);
-        onNotificationEnabledUpdated();
-
         this._setSizePosition();
 
         let updateLockState = Lang.bind(this, function() {
@@ -1468,7 +1508,7 @@ MessageTray.prototype = {
         let index = this._getSourceIndex(source);
         if (index == -1)
             return;
-            
+
         this._sources.splice(index, 1);
 
         let needUpdate = false;
@@ -1536,7 +1576,7 @@ MessageTray.prototype = {
         this._updateState();
     },
 
-    _onStatusChanged: function(presence, status) {
+    _onStatusChanged: function(status) {
         this._backFromAway = (this._userStatus == GnomeSession.PresenceStatus.IDLE && this._userStatus != status);
         this._userStatus = status;
 
@@ -1556,14 +1596,12 @@ MessageTray.prototype = {
 
     _escapeTray: function() {
         this._unlock();
-        this._pointerInTray = false;
         this._updateNotificationTimeout(0);
         this._updateState();
     },
 
     // All of the logic for what happens when occurs here; the various
-    // event handlers merely update variables such as
-    // 'this._pointerInTray', 'this._summaryState', etc, and
+    // event handlers merely update variables and
     // _updateState() figures out what (if anything) needs to be done
     // at the present time.
     _updateState: function() {
@@ -1574,9 +1612,7 @@ MessageTray.prototype = {
 
         let notificationExpired = (this._notificationTimeoutId == 0 &&
                 !(this._notification && this._notification.urgency == Urgency.CRITICAL) &&
-                !this._pointerInTray &&
-                !this._locked &&
-                !(this._pointerInKeyboard && notificationExpanded)
+                !this._locked
             ) || this._notificationRemoved;
         let canShowNotification = notificationsPending && this._notificationsEnabled;
 
@@ -1589,6 +1625,9 @@ MessageTray.prototype = {
                     this._notification = this._notificationQueue.shift();
                     if (AppletManager.get_role_provider_exists(AppletManager.Roles.NOTIFICATIONS)) {
                         this.emit('notify-applet-update', this._notification);
+                    } else {
+                        this._notification.destroy(NotificationDestroyedReason.DISMISSED);
+                        this._notification = null;
                     }
                 }
             }
@@ -1621,8 +1660,6 @@ MessageTray.prototype = {
     },
 
     _showNotification: function() {
-        this._notificationTimeoutId = 1; // this prevents a race condition with the messagetray wanting
-                                         // to hide a notification before it's done showing it, when updating from applet
         this._notification = this._notificationQueue.shift();
         if (this._notification.actor._parent_container) {
             this._notification.collapseCompleted();
@@ -1631,27 +1668,70 @@ MessageTray.prototype = {
         this._notificationClickedId = this._notification.connect('done-displaying',
                                                                  Lang.bind(this, this._escapeTray));
         this._notificationBin.child = this._notification.actor;
-        this._notificationBin.opacity = 0;        
+        this._notificationBin.opacity = 0;
+
         let monitor = Main.layoutManager.primaryMonitor;
-        this._notificationBin.y = monitor.y + (Main.panel.actor.get_height()+5) * 2; // Notifications appear from here (for the animation)
-        let margin = this._notification._table.get_theme_node().get_length('margin-from-right-edge-of-screen');                
-        this._notificationBin.x = monitor.x + monitor.width - this._notification._table.width - margin;
+        let topPanel = Main.panelManager.getPanel(0, 0);
+        let bottomPanel = Main.panelManager.getPanel(0, 1);
+        let rightPanel = Main.panelManager.getPanel(0, 3);
+        let topGap = 10;
+        let bottomGap = 10;
+        let rightGap = 0;
+
+        if (rightPanel) {
+            rightGap += rightPanel.actor.get_width();
+        }
+
+        if (!this.bottomPosition) {
+            if (topPanel) {
+                topGap += topPanel.actor.get_height();
+            }
+            this._notificationBin.y = monitor.y + topGap; // Notifications appear from here (for the animation)
+        }
+
+        let margin = this._notification._table.get_theme_node().get_length('margin-from-right-edge-of-screen');
+        this._notificationBin.x = monitor.x + monitor.width - this._notification._table.width - margin - rightGap;
+        if (!this._notification.silent || this._notification.urgency >= Urgency.HIGH) {
+            Main.soundManager.play('notification');
+        }
+        if (this._notification.urgency == Urgency.CRITICAL) {
+            Main.layoutManager._chrome.modifyActorParams(this._notificationBin, { visibleInFullscreen: true });
+        } else {
+            Main.layoutManager._chrome.modifyActorParams(this._notificationBin, { visibleInFullscreen: false });
+        }
         this._notificationBin.show();
 
+        if (this.bottomPosition) {
+            if (bottomPanel) {
+                bottomGap += bottomPanel.actor.get_height();
+            }
+            let getBottomPositionY = () => {
+                return monitor.y + monitor.height - this._notificationBin.height - bottomGap;
+            };
+            let shouldReturn = false;
+            let initialY = getBottomPositionY();
+            // For multi-line notifications, the correct height will not be known until the notification is done animating,
+            // so this will set _notificationBin.y when queue-redraw is emitted, and return early if the  height decreases
+            // to prevent unnecessary property setting.
+            this.bottomPositionSignal = this._notificationBin.connect('queue-redraw', () => {
+                if (shouldReturn) {
+                    return;
+                }
+                this._notificationBin.y = getBottomPositionY();
+                if (initialY > this._notificationBin.y) {
+                    shouldReturn = true;
+                }
+            });
+        }
+
         this._updateShowingNotification();
 
         let [x, y, mods] = global.get_pointer();
-        // We save the position of the mouse at the time when we started showing the notification
-        // in order to determine if the notification popped up under it. We make that check if
-        // the user starts moving the mouse and _onTrayHoverChanged() gets called. We don't
-        // expand the notification if it just happened to pop up under the mouse unless the user
-        // explicitly mouses away from it and then mouses back in.
-        this._showNotificationMouseX = x;
-        this._showNotificationMouseY = y;
-        // We save the y coordinate of the mouse at the time when we started showing the notification
-        // and then we update it in _notifiationTimeout() if the mouse is moving towards the
-        // notification. We don't pop down the notification if the mouse is moving towards it.
-        this._lastSeenMouseY = y;
+        // We save the distance of the mouse to the notification at the time
+        // when we started showing the it and then we update it in
+        // _notifiationTimeout() if the mouse is moving towards the notification.
+        // We don't pop down the notification if the mouse is moving towards it.
+        this._lastSeenMouseDistance = Math.abs(this._notificationBin.y - y);
     },
 
     _updateShowingNotification: function() {
@@ -1679,13 +1759,20 @@ MessageTray.prototype = {
                             onCompleteScope: this
                           };
         let monitor = Main.layoutManager.primaryMonitor;
-        if (!this._notification.expanded)        	 
-            tweenParams.y = monitor.y + (Main.panel.actor.get_height()+5);
+        let panel = Main.panelManager.getPanel(0, 0); // We only want the top panel in monitor 0
+        let height = 5;
+        if (panel)
+            height += panel.actor.get_height();
+
+        if (!this._notification.expanded)
+            tweenParams.y = monitor.y + height;
 
         this._tween(this._notificationBin, '_notificationState', State.SHOWN, tweenParams);
    },
 
     _showNotificationCompleted: function() {
+        this._updateNotificationTimeout(0);
+
         if (this._notification.urgency != Urgency.CRITICAL) {
             this._updateNotificationTimeout(NOTIFICATION_TIMEOUT * 1000);
         } else if (AppletManager.get_role_provider_exists(AppletManager.Roles.NOTIFICATIONS)) {
@@ -1706,12 +1793,13 @@ MessageTray.prototype = {
 
     _notificationTimeout: function() {
         let [x, y, mods] = global.get_pointer();
-        if (y > this._lastSeenMouseY + 10) {
+        let distance = Math.abs(this._notificationBin.y - y);
+        if (distance < this._lastSeenMouseDistance - 50 || this._notification.actor.hover) {
             // The mouse is moving towards the notification, so don't
             // hide it yet. (We just create a new timeout (and destroy
-            // the old one) each time because the bookkeeping is
-            // simpler.)
-            this._lastSeenMouseY = y;
+            // the old one) each time because the bookkeeping is simpler.)
+
+            this._lastSeenMouseDistance = distance;
             this._updateNotificationTimeout(1000);
         } else {
             this._notificationTimeoutId = 0;
@@ -1728,14 +1816,22 @@ MessageTray.prototype = {
             this._notificationExpandedId = 0;
         }
 
-        this._tween(this._notificationBin, '_notificationState', State.HIDDEN,
-                    { y: Main.layoutManager.primaryMonitor.y,
-                      opacity: 0,
-                      time: ANIMATION_TIME,
-                      transition: 'easeOutQuad',
-                      onComplete: this._hideNotificationCompleted,
-                      onCompleteScope: this
-                    });
+        let y = Main.layoutManager.primaryMonitor.y;
+        if (this.bottomPosition) {
+            if (this.bottomPositionSignal) {
+                this._notificationBin.disconnect(this.bottomPositionSignal);
+            }
+            y += Main.layoutManager.primaryMonitor.height - this._notificationBin.height;
+        }
+
+        this._tween(this._notificationBin, '_notificationState', State.HIDDEN, {
+            y,
+            opacity: 0,
+            time: ANIMATION_TIME,
+            transition: 'easeOutQuad',
+            onComplete: this._hideNotificationCompleted,
+            onCompleteScope: this
+        });
     },
 
     _hideNotificationCompleted: function() {
@@ -1749,7 +1845,7 @@ MessageTray.prototype = {
             this.emit('notify-applet-update', notification);
         } else {
             if (notification.isTransient)
-                notification.destroy(NotificationDestroyedReason.EXPIRED);  
+                notification.destroy(NotificationDestroyedReason.EXPIRED);
         }
         this._notification = null;
         this._notificationRemoved = false;
@@ -1772,17 +1868,21 @@ MessageTray.prototype = {
         let expandedY = this._notification.actor.height - this._notificationBin.height;
         // Don't animate the notification to its new position if it has shrunk:
         // there will be a very visible "gap" that breaks the illusion.
-        
+
         // This isn't really working at the moment, but it was just crashing before
         // if it encountered a critical notification.  expandedY is always 0.  For now
         // just make sure it's not covering the top panel if there is one.
-        
+
         let monitor = Main.layoutManager.primaryMonitor;
-        let newY = monitor.y + (Main.panel.actor.get_height()+5);
+        let panel = Main.panelManager.getPanel(0, 0); // We only want the top panel in monitor 0
+        let height = 5;
+        if (panel)
+            height += panel.actor.get_height();
+        let newY = monitor.y + height;
 
         if (this._notificationBin.y < expandedY)
             this._notificationBin.y = expandedY;
-        else if (this._notification.y != expandedY)
+        else if (this._notification.actor.y != expandedY)
             this._tween(this._notificationBin, '_notificationState', State.SHOWN,
                         { y: newY,
                           time: ANIMATION_TIME,
