@@ -16,6 +16,7 @@
 
 #include <clutter/x11/clutter-x11.h>
 #include <X11/extensions/Xfixes.h>
+#include "st.h"
 
 typedef enum {
   RECORDER_STATE_CLOSED,
@@ -45,15 +46,18 @@ struct _CinnamonRecorder {
   int count; /* How many times the recording has been started */
 
   ClutterStage *stage;
+  gboolean custom_area;
+  cairo_rectangle_int_t area;
   int stage_width;
   int stage_height;
 
   gboolean have_pointer;
   int pointer_x;
   int pointer_y;
-  
-  guint height_adjust; // Y adjustment from bottom panel, if any
-  
+
+  guint vertical_adjust; // Y adjustment from bottom panel, if any
+  guint horizontal_adjust; // X adjustment to position on right edge of primary monitor
+
   gboolean have_xfixes;
   int xfixes_event_base;
 
@@ -202,12 +206,12 @@ create_recording_icon (void)
 
   cairo_destroy (cr);
 
-  texture = cogl_texture_new_from_data (32, 32,
-                                        COGL_TEXTURE_NONE,
-                                        CLUTTER_CAIRO_FORMAT_ARGB32,
-                                        COGL_PIXEL_FORMAT_ANY,
-                                        cairo_image_surface_get_stride (surface),
-                                        cairo_image_surface_get_data (surface));
+  texture = st_cogl_texture_new_from_data_wrapper (32, 32,
+                                                   COGL_TEXTURE_NONE,
+                                                   CLUTTER_CAIRO_FORMAT_ARGB32,
+                                                   COGL_PIXEL_FORMAT_ANY,
+                                                   cairo_image_surface_get_stride (surface),
+                                                   cairo_image_surface_get_data (surface));
   cairo_surface_destroy (surface);
 
   return texture;
@@ -263,24 +267,23 @@ recorder_repaint_hook (gpointer data)
 static void
 cinnamon_recorder_init (CinnamonRecorder *recorder)
 {
+  GdkRectangle work_rect, geo_rect;
+  GdkScreen *screen;
+  gint primary;
+
   /* Calling gst_init() is a no-op if GStreamer was previously initialized */
   gst_init (NULL, NULL);
 
   cinnamon_recorder_src_register ();
-  
-  // Get bottom panel height from settings so we don't overlap it
-  // with our recording indicators.
-  GVariant *variant = NULL;
-  GSettings *desktop_settings;
-  
-  desktop_settings = g_settings_new (SCHEMA_CINNAMON);
-  variant = g_settings_get_value (desktop_settings, PANEL_HEIGHT_KEY);
-  g_variant_get (variant, "i", &recorder->height_adjust);
-  
-  g_variant_unref (variant);
-  g_object_unref (desktop_settings);
-  
-  
+
+  screen = gdk_screen_get_default ();
+  primary = gdk_screen_get_primary_monitor (screen);
+  gdk_screen_get_monitor_workarea (screen, primary, &work_rect);
+  gdk_screen_get_monitor_geometry (screen, primary, &geo_rect);
+
+  recorder->vertical_adjust = (geo_rect.y + geo_rect.height) - (work_rect.y + work_rect.height);
+  recorder->horizontal_adjust = work_rect.x + work_rect.width;
+
   recorder->recording_icon = create_recording_icon ();
   recorder->memory_target = get_memory_target();
 
@@ -292,18 +295,6 @@ static void
 cinnamon_recorder_finalize (GObject  *object)
 {
   CinnamonRecorder *recorder = CINNAMON_RECORDER (object);
-  GSList *l;
-
-  for (l = recorder->pipelines; l; l = l->next)
-    {
-      RecorderPipeline *pipeline = l->data;
-
-      /* Remove the back-reference. The pipeline will be freed
-       * when it finishes. (Or when the process exits, but that's
-       * out of our control.)
-       */
-      pipeline->recorder = NULL;
-    }
 
   if (recorder->update_memory_used_timeout) {
     g_source_remove (recorder->update_memory_used_timeout);
@@ -456,10 +447,10 @@ recorder_draw_cursor (CinnamonRecorder *recorder,
   /* We don't show a cursor unless the hot spot is in the frame; this
    * means that sometimes we aren't going to draw a cursor even when
    * there is a little bit overlapping within the stage */
-  if (recorder->pointer_x < 0 ||
-      recorder->pointer_y < 0 ||
-      recorder->pointer_x >= recorder->stage_width ||
-      recorder->pointer_y >= recorder->stage_height)
+  if (recorder->pointer_x < recorder->area.x ||
+      recorder->pointer_y < recorder->area.y ||
+      recorder->pointer_x >= recorder->area.x + recorder->area.width ||
+      recorder->pointer_y >= recorder->area.y + recorder->area.height)
     return;
 
   if (!recorder->cursor_image)
@@ -471,15 +462,15 @@ recorder_draw_cursor (CinnamonRecorder *recorder,
   gst_buffer_map (buffer, &info, GST_MAP_WRITE);
   surface = cairo_image_surface_create_for_data (info.data,
                                                  CAIRO_FORMAT_ARGB32,
-                                                 recorder->stage_width,
-                                                 recorder->stage_height,
-                                                 recorder->stage_width * 4);
+                                                 recorder->area.width,
+                                                 recorder->area.height,
+                                                 recorder->area.width * 4);
 
   cr = cairo_create (surface);
   cairo_set_source_surface (cr,
                             recorder->cursor_image,
-                            recorder->pointer_x - recorder->cursor_hot_x,
-                            recorder->pointer_y - recorder->cursor_hot_y);
+                            recorder->pointer_x - recorder->cursor_hot_x - recorder->area.x,
+                            recorder->pointer_y - recorder->cursor_hot_y - recorder->area.y);
   cairo_paint (cr);
 
   cairo_destroy (cr);
@@ -508,14 +499,14 @@ recorder_draw_buffer_meter (CinnamonRecorder *recorder)
   fill_level = MIN (60, (recorder->memory_used * 60) / recorder->memory_target);
 
   /* A hollow rectangle filled from the left to fill_level */
-  cogl_rectangle (recorder->stage_width - 64, recorder->stage_height - recorder->height_adjust - 10,
-                  recorder->stage_width - 2,  recorder->stage_height - recorder->height_adjust - 9);
-  cogl_rectangle (recorder->stage_width - 64, recorder->stage_height - recorder->height_adjust - 9,
-                  recorder->stage_width - (63 - fill_level), recorder->stage_height - recorder->height_adjust - 3);
-  cogl_rectangle (recorder->stage_width - 3,  recorder->stage_height - recorder->height_adjust - 9,
-                  recorder->stage_width - 2,  recorder->stage_height - recorder->height_adjust - 3);
-  cogl_rectangle (recorder->stage_width - 64, recorder->stage_height - recorder->height_adjust - 3,
-                  recorder->stage_width - 2,  recorder->stage_height - recorder->height_adjust - 2);
+  cogl_rectangle (recorder->horizontal_adjust - 64, recorder->stage_height - recorder->vertical_adjust - 10,
+                  recorder->horizontal_adjust - 2,  recorder->stage_height - recorder->vertical_adjust - 9);
+  cogl_rectangle (recorder->horizontal_adjust - 64, recorder->stage_height - recorder->vertical_adjust - 9,
+                  recorder->horizontal_adjust - (63 - fill_level), recorder->stage_height - recorder->vertical_adjust - 3);
+  cogl_rectangle (recorder->horizontal_adjust - 3,  recorder->stage_height - recorder->vertical_adjust - 9,
+                  recorder->horizontal_adjust - 2,  recorder->stage_height - recorder->vertical_adjust - 3);
+  cogl_rectangle (recorder->horizontal_adjust - 64, recorder->stage_height - recorder->vertical_adjust - 3,
+                  recorder->horizontal_adjust - 2,  recorder->stage_height - recorder->vertical_adjust - 2);
 }
 
 /* We want to time-stamp each frame based on the actual time it was
@@ -537,27 +528,75 @@ get_wall_time (void)
 /* Retrieve a frame and feed it into the pipeline
  */
 static void
-recorder_record_frame (CinnamonRecorder *recorder)
+recorder_record_frame (CinnamonRecorder *recorder,
+                       gboolean          paint)
 {
   GstBuffer *buffer;
-  guint8 *data;
+  ClutterCapture *captures;
+  int n_captures;
+  cairo_surface_t *image;
   guint size;
+  uint8_t *data;
+  GstMemory *memory;
+  int i;
+  GstClock *clock;
+  GstClockTime now, base_time;
 
-  size = recorder->stage_width * recorder->stage_height * 4;
-  data = g_malloc (size);
+  g_return_if_fail (recorder->current_pipeline != NULL);
+
+  /* If we get into the red zone, stop buffering new frames; 13/16 is
+  * a bit more than the 3/4 threshold for a red indicator to keep the
+  * indicator from flashing between red and yellow. */
+  if (recorder->memory_used > (recorder->memory_target * 13) / 16)
+    return;
+
+  /* Drop frames to get down to something like the target frame rate; since frames
+   * are generated with VBlank sync, we don't have full control anyways, so we just
+   * drop frames if the interval since the last frame is less than 75% of the
+   * desired inter-frame interval.
+   */
+  clock = gst_element_get_clock (recorder->current_pipeline->src);
+
+  /* If we have no clock yet, the pipeline is not yet in PLAYING */
+  if (!clock)
+    return;
+
+  base_time = gst_element_get_base_time (recorder->current_pipeline->src);
+  now = gst_clock_get_time (clock) - base_time;
+  gst_object_unref (clock);
+
+  if (GST_CLOCK_TIME_IS_VALID (recorder->start_time) &&
+      now - recorder->start_time < gst_util_uint64_scale_int (GST_SECOND, 3, 4 * recorder->framerate))
+    return;
+  recorder->start_time = now;
+
+  clutter_stage_capture (recorder->stage, paint, &recorder->area,
+                         &captures, &n_captures);
+
+  if (n_captures == 0)
+    return;
+
+  /*
+   * TODO: Deal with each capture region separately, instead of dropping
+   * anything except the first one.
+   */
+
+  image = captures[0].image;
+  data = cairo_image_surface_get_data (image);
+  size = captures[0].rect.width * captures[0].rect.height * 4;
+
+  /* TODO: Capture more than the first framebuffer. */
+  for (i = 1; i < n_captures; i++)
+    cairo_surface_destroy (captures[i].image);
+  g_free (captures);
 
   buffer = gst_buffer_new();
-  gst_buffer_insert_memory (buffer, -1,
-                            gst_memory_new_wrapped (0, data, size, 0,
-                                                    size, data, g_free));
+  memory = gst_memory_new_wrapped (0, data, size, 0, size,
+                                   image,
+                                   (GDestroyNotify) cairo_surface_destroy);
+  gst_buffer_insert_memory (buffer, -1, memory);
 
-  GST_BUFFER_PTS(buffer) = get_wall_time() - recorder->start_time;
-
-  cogl_read_pixels (0, 0,
-                    recorder->stage_width, recorder->stage_height,
-                    COGL_READ_PIXELS_COLOR_BUFFER,
-                    CLUTTER_CAIRO_FORMAT_ARGB32,
-                    data);
+  GST_BUFFER_PTS(buffer) = now;
 
   recorder_draw_cursor (recorder, buffer);
 
@@ -573,17 +612,17 @@ recorder_record_frame (CinnamonRecorder *recorder)
  * by clutter before glSwapBuffers() makes it visible to the user.
  */
 static void
-recorder_on_stage_paint (ClutterActor  *actor,
+recorder_on_stage_paint (ClutterActor     *actor,
                          CinnamonRecorder *recorder)
 {
   if (recorder->state == RECORDER_STATE_RECORDING)
     {
       if (!recorder->only_paint)
-        recorder_record_frame (recorder);
+        recorder_record_frame (recorder, FALSE);
 
       cogl_set_source_texture (recorder->recording_icon);
-      cogl_rectangle (recorder->stage_width - 32, recorder->stage_height - recorder->height_adjust - 42,
-                      recorder->stage_width,      recorder->stage_height - recorder->height_adjust - 10);
+      cogl_rectangle (recorder->horizontal_adjust - 32, recorder->stage_height - recorder->vertical_adjust - 42,
+                      recorder->horizontal_adjust,      recorder->stage_height - recorder->vertical_adjust - 10);
     }
 
   if (recorder->state == RECORDER_STATE_RECORDING || recorder->memory_used != 0)
@@ -598,6 +637,14 @@ recorder_update_size (CinnamonRecorder *recorder)
   clutter_actor_get_allocation_box (CLUTTER_ACTOR (recorder->stage), &allocation);
   recorder->stage_width = (int)(0.5 + allocation.x2 - allocation.x1);
   recorder->stage_height = (int)(0.5 + allocation.y2 - allocation.y1);
+
+  if (!recorder->custom_area)
+    {
+      recorder->area.x = 0;
+      recorder->area.y = 0;
+      recorder->area.width = recorder->stage_width;
+      recorder->area.height = recorder->stage_height;
+    }
 }
 
 static void
@@ -1062,8 +1109,8 @@ recorder_pipeline_set_caps (RecorderPipeline *pipeline)
                               "bpp", G_TYPE_INT, 32,
                               "depth", G_TYPE_INT, 24,
                               "framerate", GST_TYPE_FRACTION, pipeline->recorder->framerate, 1,
-                              "width", G_TYPE_INT, pipeline->recorder->stage_width,
-                              "height", G_TYPE_INT, pipeline->recorder->stage_height,
+                              "width", G_TYPE_INT, pipeline->recorder->area.width,
+                              "height", G_TYPE_INT, pipeline->recorder->area.height,
                               NULL);
   g_object_set (pipeline->src, "caps", caps, NULL);
   gst_caps_unref (caps);
@@ -1386,6 +1433,8 @@ recorder_pipeline_free (RecorderPipeline *pipeline)
   if (pipeline->outfile != -1)
     close (pipeline->outfile);
 
+  g_clear_object (&pipeline->recorder);
+
   g_free (pipeline);
 }
 
@@ -1399,13 +1448,13 @@ recorder_pipeline_bus_watch (GstBus     *bus,
 {
   RecorderPipeline *pipeline = data;
 
-  switch (message->type)
+  if (message->type == GST_MESSAGE_EOS)
     {
-    case GST_MESSAGE_EOS:
       recorder_pipeline_closed (pipeline);
       return FALSE; /* remove watch */
-    case GST_MESSAGE_ERROR:
-      {
+    }
+  else if (message->type == GST_MESSAGE_ERROR)
+    {
         GError *error;
 
         gst_message_parse_error (message, &error, NULL);
@@ -1413,9 +1462,6 @@ recorder_pipeline_bus_watch (GstBus     *bus,
         g_error_free (error);
         recorder_pipeline_closed (pipeline);
         return FALSE; /* remove watch */
-      }
-    default:
-      break;
     }
 
   /* Leave the watch in place */
@@ -1494,7 +1540,7 @@ recorder_open_pipeline (CinnamonRecorder *recorder)
   GstBus *bus;
 
   pipeline = g_new0(RecorderPipeline, 1);
-  pipeline->recorder = recorder;
+  pipeline->recorder = g_object_ref (recorder);
   pipeline->outfile = - 1;
 
   pipeline_description = recorder->pipeline_description;
@@ -1650,6 +1696,31 @@ cinnamon_recorder_set_pipeline (CinnamonRecorder *recorder,
   recorder_set_pipeline (recorder, pipeline);
 }
 
+void
+cinnamon_recorder_set_area (CinnamonRecorder *recorder,
+                            int               x,
+                            int               y,
+                            int               width,
+                            int               height)
+{
+  g_return_if_fail (CINNAMON_IS_RECORDER (recorder));
+
+  recorder->custom_area = TRUE;
+  recorder->area.x = CLAMP (x, 0, recorder->stage_width);
+  recorder->area.y = CLAMP (y, 0, recorder->stage_height);
+  recorder->area.width = CLAMP (width,
+                                0, recorder->stage_width - recorder->area.x);
+  recorder->area.height = CLAMP (height,
+                                 0, recorder->stage_height - recorder->area.y);
+
+  /* This breaks the recording but tweaking the GStreamer pipeline a bit
+   * might make it work, at least if the codec can handle a stream where
+   * the frame size changes in the middle.
+   */
+  if (recorder->current_pipeline)
+    recorder_pipeline_set_caps (recorder->current_pipeline);
+}
+
 /**
  * cinnamon_recorder_record:
  * @recorder: the #CinnamonRecorder
@@ -1662,12 +1733,12 @@ cinnamon_recorder_set_pipeline (CinnamonRecorder *recorder,
  * recording failed to start.
  *
  * An extra reference count is added to the recorder if recording
- * is succesfully started; the recording object will not be freed
+ * is successfully started; the recording object will not be freed
  * until recording is stopped even if the creator no longer holds
  * a reference. Recording is automatically stopped if the stage
  * is destroyed.
  *
- * Return value: %TRUE if recording was succesfully started
+ * Return value: %TRUE if recording was successfully started
  */
 gboolean
 cinnamon_recorder_record (CinnamonRecorder *recorder)
@@ -1728,7 +1799,7 @@ cinnamon_recorder_pause (CinnamonRecorder *recorder)
   /* We want to record one more frame since some time may have
    * elapsed since the last frame
    */
-  clutter_actor_paint (CLUTTER_ACTOR (recorder->stage));
+  recorder_record_frame (recorder, TRUE);
 
   if (recorder->filename_has_count)
     recorder_close_pipeline (recorder);
