@@ -10,6 +10,9 @@
 #include <langinfo.h>
 #endif
 
+#define DESKTOP_SCHEMA "org.cinnamon.desktop.interface"
+#define FIRST_WEEKDAY_KEY "first-day-of-week"
+
 #include <libxml/parser.h>
 #include <libxml/tree.h>
 #include <libxml/xmlmemory.h>
@@ -68,6 +71,7 @@ cinnamon_util_get_file_display_name_if_mount (GFile *file)
       if (!ret && g_file_equal (file, compare))
         ret = g_mount_get_name (mount);
       g_object_unref (mount);
+      g_object_unref (compare);
     }
   g_list_free (mounts);
   g_object_unref (monitor);
@@ -88,6 +92,7 @@ cinnamon_util_get_file_display_for_common_files (GFile *file)
        * nemo */
       return g_strdup (_("Home"));
     }
+  g_object_unref (compare);
 
   compare = g_file_new_for_path ("/");
   if (g_file_equal (file, compare))
@@ -176,6 +181,7 @@ cinnamon_util_get_file_icon_if_mount (GFile *file)
           ret = g_mount_get_icon (mount);
         }
       g_object_unref (mount);
+      g_object_unref (compare);
     }
   g_list_free (mounts);
   g_object_unref (monitor);
@@ -259,7 +265,7 @@ cinnamon_util_get_label_for_uri (const char *text_uri)
         label = cinnamon_util_get_file_description (file);
       if (!label)
         label = cinnamon_util_get_file_display_name (file, TRUE);
-        g_object_unref (file);
+      g_object_unref (file);
 
       return label;
     }
@@ -319,6 +325,7 @@ cinnamon_util_get_icon_for_uri (const char *text_uri)
   GFile *file;
   GFileInfo *info;
   GIcon *retval;
+  const char *custom_icon;
 
   /* Here's what we do:
    *  + check for known file: URI
@@ -347,7 +354,10 @@ cinnamon_util_get_icon_for_uri (const char *text_uri)
 
   retval = cinnamon_util_get_file_icon_if_mount (file);
   if (retval)
-    return retval;
+    {
+      g_object_unref (file);
+      return retval;
+    }
 
   /* gvfs doesn't give us a nice icon for subfolders of the trash, so
    * overriding */
@@ -360,22 +370,38 @@ cinnamon_util_get_icon_for_uri (const char *text_uri)
       file = root;
     }
 
-  info = g_file_query_info (file, "standard::icon", G_FILE_QUERY_INFO_NONE,
+  info = g_file_query_info (file, "standard::icon,metadata::custom-icon", G_FILE_QUERY_INFO_NONE,
                             NULL, NULL);
   g_object_unref (file);
 
   if (!info)
-    return g_themed_icon_new ("gtk-file");
+    return g_themed_icon_new ("text-x-preview");
+
+  custom_icon = g_file_info_get_attribute_string (info, "metadata::custom-icon");
+
+  if (custom_icon)
+    {
+      GFile *icon_file = g_file_new_for_uri (custom_icon);
+
+      retval = g_file_icon_new (icon_file);
+
+      g_object_unref (icon_file);
+    }
+
+  if (!retval)
+    {
+      retval = g_file_info_get_icon (info);
+
+      if (retval)
+        g_object_ref (retval);
+    }
 
-  retval = g_file_info_get_icon (info);
-  if (retval)
-    g_object_ref (retval);
   g_object_unref (info);
 
   if (retval)
     return retval;
 
-  return g_themed_icon_new ("gtk-file");
+  return g_themed_icon_new ("text-x-preview");
 }
 
 static void
@@ -443,7 +469,7 @@ cinnamon_util_get_transformed_allocation (ClutterActor    *actor,
    */
   ClutterVertex v[4];
   gfloat x_min, x_max, y_min, y_max;
-  gint i;
+  guint i;
 
   g_return_if_fail (CLUTTER_IS_ACTOR (actor));
 
@@ -473,20 +499,6 @@ cinnamon_util_get_transformed_allocation (ClutterActor    *actor,
   box->y2 = y_max;
 }
 
-char *
-cinnamon_util_normalize_and_casefold (const char *str)
-{
-  char *normalized, *result;
-
-  if (str == NULL)
-    return NULL;
-
-  normalized = g_utf8_normalize (str, -1, G_NORMALIZE_ALL);
-  result = g_utf8_casefold (normalized, -1);
-  g_free (normalized);
-  return result;
-}
-
 /**
  * cinnamon_util_format_date:
  * @format: a strftime-style string format, as parsed by
@@ -538,9 +550,9 @@ cinnamon_util_format_date (const char *format,
  */
 /* Copied from gtkcalendar.c */
 int
-cinnamon_util_get_week_start ()
+cinnamon_util_get_week_start (void)
 {
-  int week_start;
+
 #ifdef HAVE__NL_TIME_FIRST_WEEKDAY
   union { unsigned int word; char *string; } langinfo;
   int week_1stday = 0;
@@ -550,6 +562,15 @@ cinnamon_util_get_week_start ()
   char *gtk_week_start;
 #endif
 
+  /* Try to get first weekday from gsettings */
+  /* If the value from gsettings is not in the range 0-6,
+   * continue to get the locale's first weekday */
+  GSettings *settings = g_settings_new (DESKTOP_SCHEMA);
+  int week_start = g_settings_get_int (settings, FIRST_WEEKDAY_KEY);
+  g_object_unref (settings);
+
+  if (0 <= week_start && week_start < 7) return week_start;
+
 #ifdef HAVE__NL_TIME_FIRST_WEEKDAY
   langinfo.string = nl_langinfo (_NL_TIME_FIRST_WEEKDAY);
   first_weekday = langinfo.string[0];
@@ -575,8 +596,8 @@ cinnamon_util_get_week_start ()
 
   if (week_start < 0 || week_start > 6)
     {
-      g_warning ("Whoever translated calendar:week_start:0 for GTK+ "
-                 "did so wrongly.\n");
+      g_warning ("calendar:week_start:0 for GTK+ "
+                 "was translated wrongly.\n");
       week_start = 0;
     }
 #endif
@@ -656,6 +677,8 @@ cinnamon_write_string_to_stream (GOutputStream *stream,
  * invalid content.
  *
  * Returns: (transfer full): File contents
+ *
+ * Deprecated: 3.1
  */
 char *
 cinnamon_get_file_contents_utf8_sync (const char *path,
@@ -678,6 +701,101 @@ cinnamon_get_file_contents_utf8_sync (const char *path,
   return contents;
 }
 
+typedef struct
+{
+    CinnamonFileContentsCallback callback;
+    gpointer user_data;
+} CinnamonFileContentsCallbackData;
+
+static void
+get_file_contents_utf8_thread (GTask        *task,
+                               gpointer      source_object,
+                               gpointer      task_data,
+                               GCancellable *cancellable)
+{
+    gchar *contents = NULL;
+    GError *error = NULL;
+
+    const gchar *path = (const gchar *) task_data;
+
+    contents = cinnamon_get_file_contents_utf8_sync (path, &error);
+
+    if (error)
+      {
+        g_task_return_error (task, error);
+        return;
+      }
+
+    g_task_return_pointer (task, contents, g_free);
+}
+
+static void
+get_file_contents_utf8_task_finished (GObject      *source,
+                                      GAsyncResult *result,
+                                      gpointer      user_data)
+{
+    CinnamonFileContentsCallbackData *data = (CinnamonFileContentsCallbackData *) user_data;
+
+    GError *error = NULL;
+    gchar *contents = NULL;
+
+    contents = g_task_propagate_pointer (G_TASK (result), &error);
+
+    if (error)
+      {
+        g_printerr ("cinnamon_get_file_contents_utf8 failed: %s\n", error->message);
+        g_clear_error (&error);
+      }
+
+    (* data->callback) (contents, data->user_data);
+
+    g_clear_pointer (&contents, g_free);
+    g_slice_free (CinnamonFileContentsCallbackData, data);
+}
+
+/**
+ * cinnamon_get_file_contents_utf8:
+ * @path: UTF-8 encoded filename path
+ * @callback: (scope async): The callback to call when finished
+ * @user_data: (closure): data to pass with the callback
+ *
+ * Asynchronously load the contents of a file as a NUL terminated
+ * string, validating it as UTF-8.  Embedded NUL characters count as
+ * invalid content.
+ **/
+void
+cinnamon_get_file_contents_utf8         (const char                   *path,
+                                         CinnamonFileContentsCallback  callback,
+                                         gpointer                      user_data)
+{
+  gchar *async_path;
+  GTask *task;
+  CinnamonFileContentsCallbackData *data;
+
+  if (path == NULL || callback == NULL)
+    {
+      g_warning ("cinnamon_get_file_contents_utf8: path and callback cannot be null");
+      return;
+    }
+
+  data = g_slice_new (CinnamonFileContentsCallbackData);
+
+  data->callback = callback;
+  data->user_data = user_data;
+
+  async_path = g_strdup (path);
+
+  task = g_task_new (NULL,
+                     NULL,
+                     get_file_contents_utf8_task_finished,
+                     data);
+
+  g_task_set_task_data (task, async_path, (GDestroyNotify) g_free);
+  g_task_run_in_thread (task, get_file_contents_utf8_thread);
+
+  g_object_unref (task);
+}
+
 /**
  * cinnamon_breakpoint:
  *
