@@ -7,7 +7,7 @@
 #endif
 #include <stdlib.h>
 #include <string.h>
-#include <X11/Xlib.h>
+
 #include <clutter/clutter.h>
 #include <clutter/x11/clutter-x11.h>
 #include <dbus/dbus-glib.h>
@@ -20,7 +20,7 @@
 #include <meta/meta-plugin.h>
 #include <meta/prefs.h>
 
-#include "cinnamon-a11y.h"
+#include <atk-bridge.h>
 #include "cinnamon-global.h"
 #include "cinnamon-global-private.h"
 #include "cinnamon-perf-log.h"
@@ -32,99 +32,124 @@ extern GType gnome_cinnamon_plugin_get_type (void);
 #define MAGNIFIER_DBUS_SERVICE "org.gnome.Magnifier"
 
 static void
-cinnamon_dbus_init (gboolean replace)
+cinnamon_dbus_acquire_name (GDBusProxy *bus,
+                         guint32     request_name_flags,
+                         guint32    *request_name_result,
+                         gchar      *name,
+                         gboolean    fatal)
 {
   GError *error = NULL;
-  DBusGConnection *session;
-  DBusGProxy *bus;
+  GVariant *request_name_variant;
+
+  if (!(request_name_variant = g_dbus_proxy_call_sync (bus,
+                                                       "RequestName",
+                                                       g_variant_new ("(su)", name, request_name_flags),
+                                                       0, /* call flags */
+                                                       -1, /* timeout */
+                                                       NULL, /* cancellable */
+                                                       &error)))
+    {
+      g_printerr ("failed to acquire %s: %s\n", name, error->message);
+      g_clear_error (&error);
+      if (!fatal)
+        return;
+      exit (1);
+    }
+  g_variant_get (request_name_variant, "(u)", request_name_result);
+  g_variant_unref (request_name_variant);
+}
+
+static void
+cinnamon_dbus_acquire_names (GDBusProxy *bus,
+                          guint32     request_name_flags,
+                          gchar      *name,
+                          gboolean    fatal, ...) G_GNUC_NULL_TERMINATED;
+
+static void
+cinnamon_dbus_acquire_names (GDBusProxy *bus,
+                          guint32     request_name_flags,
+                          gchar      *name,
+                          gboolean    fatal, ...)
+{
+  va_list al;
+  guint32 request_name_result;
+  va_start (al, fatal);
+  for (;;)
+  {
+    cinnamon_dbus_acquire_name (bus,
+                             request_name_flags,
+                             &request_name_result,
+                             name, fatal);
+    name = va_arg (al, gchar *);
+    if (!name)
+      break;
+    fatal = va_arg (al, gboolean);
+  }
+  va_end (al);
+}
+
+static void
+cinnamon_dbus_init (gboolean replace)
+{
+  GDBusConnection *session;
+  GDBusProxy *bus;
+  GError *error = NULL;
   guint32 request_name_flags;
   guint32 request_name_result;
 
-  /** TODO:
-   * In the future we should use GDBus for this.  However, in
-   * order to do that, we need to port all of the JavaScript
-   * code.  Otherwise, the name will be claimed on the wrong
-   * connection.
-   */
-  session = dbus_g_bus_get (DBUS_BUS_SESSION, NULL);
+  session = g_bus_get_sync (G_BUS_TYPE_SESSION, NULL, &error);
 
-  bus = dbus_g_proxy_new_for_name (session,
-                                   DBUS_SERVICE_DBUS,
-                                   DBUS_PATH_DBUS,
-                                   DBUS_INTERFACE_DBUS);
+  if (error) {
+    g_printerr ("Failed to connect to session bus: %s", error->message);
+    exit (1);
+  }
 
-  request_name_flags = DBUS_NAME_FLAG_DO_NOT_QUEUE | DBUS_NAME_FLAG_ALLOW_REPLACEMENT;
+  bus = g_dbus_proxy_new_sync (session,
+                               G_DBUS_PROXY_FLAGS_NONE,
+                               NULL, /* interface info */
+                               "org.freedesktop.DBus",
+                               "/org/freedesktop/DBus",
+                               "org.freedesktop.DBus",
+                               NULL, /* cancellable */
+                               &error);
+
+  request_name_flags = G_BUS_NAME_OWNER_FLAGS_ALLOW_REPLACEMENT;
   if (replace)
     request_name_flags |= DBUS_NAME_FLAG_REPLACE_EXISTING;
-  if (!dbus_g_proxy_call (bus, "RequestName", &error,
-                          G_TYPE_STRING, CINNAMON_DBUS_SERVICE,
-                          G_TYPE_UINT, request_name_flags,
-                          G_TYPE_INVALID,
-                          G_TYPE_UINT, &request_name_result,
-                          G_TYPE_INVALID))
-    {
-      g_printerr ("failed to acquire org.Cinnamon: %s\n", error->message);
-      exit (1);
-    }
+
+  cinnamon_dbus_acquire_name (bus,
+                           request_name_flags,
+                           &request_name_result,
+                           CINNAMON_DBUS_SERVICE, TRUE);
   if (!(request_name_result == DBUS_REQUEST_NAME_REPLY_PRIMARY_OWNER
         || request_name_result == DBUS_REQUEST_NAME_REPLY_ALREADY_OWNER))
     {
-      g_printerr ("%s already exists on bus and --replace not specified\n",
-                  CINNAMON_DBUS_SERVICE);
+      g_printerr (CINNAMON_DBUS_SERVICE " already exists on bus and --replace not specified\n");
       exit (1);
     }
 
+  /*
+   * We always specify REPLACE_EXISTING to ensure we kill off
+   * the existing service if it was running.
+   */
+  request_name_flags |= G_BUS_NAME_OWNER_FLAGS_REPLACE;
+
+  cinnamon_dbus_acquire_names (bus,
+                            request_name_flags,
   /* Also grab org.gnome.Panel to replace any existing panel process */
-  if (!dbus_g_proxy_call (bus, "RequestName", &error, G_TYPE_STRING,
-                          "org.gnome.Panel", G_TYPE_UINT,
-                          DBUS_NAME_FLAG_REPLACE_EXISTING | request_name_flags,
-                          G_TYPE_INVALID, G_TYPE_UINT,
-                          &request_name_result, G_TYPE_INVALID))
-    {
-      g_print ("failed to acquire org.gnome.Panel: %s\n", error->message);
-      exit (1);
-    }
-
-  /* ...and the org.gnome.Magnifier service.
-   */
-  if (!dbus_g_proxy_call (bus, "RequestName", &error,
-                          G_TYPE_STRING, MAGNIFIER_DBUS_SERVICE,
-                          G_TYPE_UINT, DBUS_NAME_FLAG_REPLACE_EXISTING | request_name_flags,
-                          G_TYPE_INVALID,
-                          G_TYPE_UINT, &request_name_result,
-                          G_TYPE_INVALID))
-    {
-      g_print ("failed to acquire %s: %s\n", MAGNIFIER_DBUS_SERVICE, error->message);
-      /* Failing to acquire the magnifer service is not fatal.  Log the error,
-       * but keep going. */
-    }
-
-  /* ...and the org.freedesktop.Notifications service; we always
-   * specify REPLACE_EXISTING to ensure we kill off
-   * notification-daemon if it was running.
-   */
-  if (!dbus_g_proxy_call (bus, "RequestName", &error,
-                          G_TYPE_STRING, "org.freedesktop.Notifications",
-                          G_TYPE_UINT, DBUS_NAME_FLAG_REPLACE_EXISTING | request_name_flags,
-                          G_TYPE_INVALID,
-                          G_TYPE_UINT, &request_name_result,
-                          G_TYPE_INVALID))
-    {
-      g_print ("failed to acquire org.freedesktop.Notifications: %s\n", error->message);
-    }
-
+                            "org.gnome.Panel", TRUE,
+  /* ...and the org.gnome.Magnifier service. */
+                            MAGNIFIER_DBUS_SERVICE, FALSE,
+  /* ...and the org.freedesktop.Notifications service. */
+                            "org.freedesktop.Notifications", FALSE,
+                            NULL);
   /* ...and the on-screen keyboard service */
-  if (!dbus_g_proxy_call (bus, "RequestName", &error,
-                          G_TYPE_STRING, "org.gnome.Caribou.Keyboard",
-                          G_TYPE_UINT, DBUS_NAME_FLAG_REPLACE_EXISTING,
-                          G_TYPE_INVALID,
-                          G_TYPE_UINT, &request_name_result,
-                          G_TYPE_INVALID))
-    {
-      g_print ("failed to acquire org.gnome.Caribou.Keyboard: %s\n", error->message);
-    }
-
+  cinnamon_dbus_acquire_name (bus,
+                           DBUS_NAME_FLAG_REPLACE_EXISTING,
+                           &request_name_result,
+                           "org.gnome.Caribou.Keyboard", FALSE);
   g_object_unref (bus);
+  g_object_unref (session);
 }
 
 static void
@@ -146,6 +171,22 @@ malloc_statistics_callback (CinnamonPerfLog *perf_log,
 #endif
 }
 
+static void
+cinnamon_a11y_init (void)
+{
+  cally_accessibility_init ();
+
+  if (clutter_get_accessibility_enabled () == FALSE)
+    {
+      g_warning ("Accessibility: clutter has no accessibility enabled"
+                 " skipping the atk-bridge load");
+    }
+  else
+    {
+      atk_bridge_adaptor_init (NULL, NULL);
+    }
+}
+
 static void
 cinnamon_perf_log_init (void)
 {
@@ -165,7 +206,7 @@ cinnamon_perf_log_init (void)
                                    "i");
   cinnamon_perf_log_define_statistic (perf_log,
                                    "malloc.usedSize",
-                                   "Amount of malloc'ed memory currently in use",
+                                   "Amount of allocated memory currently in use",
                                    "i");
 
   cinnamon_perf_log_add_statistics_callback (perf_log,
@@ -208,7 +249,7 @@ center_pointer_on_screen ()
   Display *dpy;
   Window root_window;
   Screen *screen;
-  
+
   dpy = XOpenDisplay(0);
   root_window = XRootWindow(dpy, 0);
   XSelectInput(dpy, root_window, KeyReleaseMask);
@@ -224,7 +265,7 @@ main (int argc, char **argv)
   GError *error = NULL;
   int ecode;
   g_setenv ("CLUTTER_DISABLE_XINPUT", "1", TRUE);
-  g_type_init ();
+  g_setenv ("CLUTTER_BACKEND", "x11", TRUE);
 
   bindtextdomain (GETTEXT_PACKAGE, LOCALEDIR);
   bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
@@ -242,7 +283,7 @@ main (int argc, char **argv)
 
   g_option_context_free (ctx);
 
-  meta_plugin_type_register (gnome_cinnamon_plugin_get_type ());
+  meta_plugin_manager_set_plugin_type (gnome_cinnamon_plugin_get_type ());
 
   /* Prevent meta_init() from causing gtk to load gail and at-bridge */
   g_setenv ("NO_GAIL", "1", TRUE);
@@ -251,6 +292,7 @@ main (int argc, char **argv)
   g_unsetenv ("NO_GAIL");
   g_unsetenv ("NO_AT_BRIDGE");
   g_unsetenv ("CLUTTER_DISABLE_XINPUT");
+  g_unsetenv ("CLUTTER_BACKEND");
 
   /* FIXME: Add gjs API to set this stuff and don't depend on the
    * environment.  These propagate to child processes.
@@ -268,18 +310,16 @@ main (int argc, char **argv)
   cinnamon_perf_log_init ();
 
   g_irepository_prepend_search_path (CINNAMON_PKGLIBDIR);
-#if HAVE_BLUETOOTH
-  g_irepository_prepend_search_path (BLUETOOTH_DIR);
-#endif
 
   /* Disable debug spew from various libraries */
-  g_log_set_handler ("Gvc", G_LOG_LEVEL_DEBUG,
+  g_log_set_handler ("Cvc", G_LOG_LEVEL_DEBUG,
                      muted_log_handler, NULL);
   g_log_set_handler ("AccountsService", G_LOG_LEVEL_DEBUG,
                      muted_log_handler, NULL);
   g_log_set_handler ("Bluetooth", G_LOG_LEVEL_DEBUG | G_LOG_LEVEL_MESSAGE,
                      muted_log_handler, NULL);
-
+  g_log_set_handler ("XApp", G_LOG_LEVEL_DEBUG,
+                     muted_log_handler, NULL);
   /* Initialize the global object */
   _cinnamon_global_init (NULL);
 
